@@ -11,7 +11,7 @@ from aiohttp.client_reqrep import ClientResponse
 from asynctest import fail_on, skipIf
 from asynctest.case import TestCase
 from ddt import ddt, data
-
+from asyncio import CancelledError, TimeoutError
 try:
     from aiohttp.errors import (
         ClientConnectionError,
@@ -94,6 +94,15 @@ class AIOResponsesTestCase(TestCase):
         self.assertEqual(response.headers['Connection'], 'keep-alive')
         self.assertEqual(response.headers[hdrs.CONTENT_TYPE], 'text/html')
 
+    @aioresponses()
+    @asyncio.coroutine
+    def test_returned_response_cookies(self, m):
+        m.get(self.url,
+              headers={'Set-Cookie': 'cookie=value'})
+        response = yield from self.session.get(self.url)
+
+        self.assertEqual(response.cookies['cookie'].value, 'value')
+
     @aioresponses()
     @asyncio.coroutine
     def test_returned_response_raw_headers(self, m):
@@ -223,11 +232,42 @@ class AIOResponsesTestCase(TestCase):
                 yield from self.session.get('http://example.com/foo')
 
     @asyncio.coroutine
-    def test_raising_custom_error(self):
+    def test_raising_exception(self):
         with aioresponses() as aiomock:
-            aiomock.get(self.url, exception=HttpProcessingError(message='foo'))
+            url = 'http://example.com/Exception'
+            aiomock.get(url, exception=Exception)
+            with self.assertRaises(Exception):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/Exception_object'
+            aiomock.get(url, exception=Exception())
+            with self.assertRaises(Exception):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/BaseException'
+            aiomock.get(url, exception=BaseException)
+            with self.assertRaises(BaseException):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/BaseException_object'
+            aiomock.get(url, exception=BaseException())
+            with self.assertRaises(BaseException):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/CancelError'
+            aiomock.get(url, exception=CancelledError)
+            with self.assertRaises(CancelledError):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/TimeoutError'
+            aiomock.get(url, exception=TimeoutError)
+            with self.assertRaises(TimeoutError):
+                yield from self.session.get(url)
+
+            url = 'http://example.com/HttpProcessingError'
+            aiomock.get(url, exception=HttpProcessingError(message='foo'))
             with self.assertRaises(HttpProcessingError):
-                yield from self.session.get(self.url)
+                yield from self.session.get(url)
 
     @asyncio.coroutine
     def test_multiple_requests(self):
@@ -340,6 +380,23 @@ class AIOResponsesTestCase(TestCase):
             self.assertEqual(api.status, 200)
             self.assertEqual(ext.status, 201)
 
+    @asyncio.coroutine
+    def test_pass_through_with_origin_params(self):
+        external_api = 'http://httpbin.org/get'
+
+        @asyncio.coroutine
+        def doit(params):
+            # we have to hit actual url,
+            # otherwise we do not test pass through option properly
+            ext_rep = (yield from self.session.get(URL(external_api), params=params))
+            return ext_rep
+
+        with aioresponses(passthrough=[external_api]) as m:
+            params = {'foo': 'bar'}
+            ext = yield from doit(params=params)
+            self.assertEqual(ext.status, 200)
+            self.assertEqual(str(ext.url), 'http://httpbin.org/get?foo=bar')
+
     @aioresponses()
     @asyncio.coroutine
     def test_custom_response_class(self, m):
@@ -529,6 +586,24 @@ class AIOResponseRedirectTest(TestCase):
         self.assertEqual(len(response.history), 1)
         self.assertEqual(str(response.history[0].url), self.url)
 
+    @aioresponses()
+    @asyncio.coroutine
+    def test_post_redirect_followed(self, rsps):
+        rsps.post(
+            self.url,
+            status=307,
+            headers={"Location": "https://httpbin.org"},
+        )
+        rsps.get("https://httpbin.org")
+        response = yield from self.session.post(
+            self.url, allow_redirects=True
+        )
+        self.assertEqual(response.status, 200)
+        self.assertEqual(str(response.url), "https://httpbin.org")
+        self.assertEqual(response.method, "get")
+        self.assertEqual(len(response.history), 1)
+        self.assertEqual(str(response.history[0].url), self.url)
+
     @aioresponses()
     @asyncio.coroutine
     def test_redirect_missing_mocked_match(self, rsps):
